/* (C) Universitaet Passau 1986-1991 */
%{/**********************************************************************/
/*									*/
/*				stdsanner.l				*/
/*									*/
/************************************************************************/

#include "sparser.tab.h"
#include <string.h>
#include <malloc.h>

const int SCANNER_KEY_MAXLEN=1000;
const char ESCAPE_CHARACTER='\\';

char	*remove_escape_characters_from_text (char *text, int length);
int	scanner_input_file_linenumber;
char	*text_without_escapes = 0;

%}

digit		[0-9]
number		-?{digit}+
floatnumber	-?{digit}+\.{digit}+

left_brace		\{
dollar			\$
no_dollar		[^\$]
right_brace		\}
no_right_brace		[^\}]
escaped_brace		(\\{right_brace}|\\{left_brace})

begin_graphed_attributes	{left_brace}{dollar}
end_graphed_attributes	{dollar}{right_brace}

quotation_mark			\"
no_quotation_mark		[^\"]
escaped_character		\\.
no_quotation_mark_or_backslash	[^"\\]
white_space			[ \t]
newline				"\n"

hash_mark	\#
anything	.*

%%
{number}	{
			yylval.nr = atoi (yytext);
			return SCANNER_NUMBER;
		}

{floatnumber}	{
			yylval.floatnr = atof (yytext);
			return SCANNER_FLOATNUMBER;
		}

{quotation_mark}({escaped_character}|{no_quotation_mark_or_backslash})*{quotation_mark}	{

			if (text_without_escapes != 0) {
				/* Clean up heap */
				free (text_without_escapes);
			}

			text_without_escapes =
				remove_escape_characters_from_text (
					&(yytext[1]),
					yyleng-2);
			yylval.text = text_without_escapes;

			return SCANNER_IDENTIFIER;
		}

{begin_graphed_attributes}	{
			return SCANNER_BEGIN_GRAPHED_ATTRIBUTES;
		}
{end_graphed_attributes}	{
			return SCANNER_END_GRAPHED_ATTRIBUTES;
		}


GRAPH		return SCANNER_GRAPH;
END		return SCANNER_END;
"="		return ('=');
";"		return (';');
"["		return ('[');
"]"		return (']');
GRAPHS		return SCANNER_GRAPHS;
DIRECTED	return SCANNER_DIRECTED;
UNDIRECTED	return SCANNER_UNDIRECTED;
","		return (',');

[a-zA-Z_:][a-zA-Z0-9_:]*	{

			static	char	id[SCANNER_KEY_MAXLEN+1];
			int		length;
						
			length = yyleng < SCANNER_KEY_MAXLEN ?
			         yyleng : SCANNER_KEY_MAXLEN;
			strncpy (id, yytext, length);
			id[length] = '\0';

			yylval.text = id;
			
			return SCANNER_KEY;
		}


{newline}	{ 
			scanner_input_file_linenumber++;
			/* Ansonsten ueberlesen			*/
		}
{white_space}	;

{hash_mark}{anything}	/* New Style Comment */;

.		{
			return yytext[0];
			/*  = alles hier nicht aufgefuehrte	*/
		}
%%

/* Taken from GraphEd 4.0 source code */

char	*remove_escape_characters_from_text (char *text, int length)
{
	register	int	i,j;
	register	int	length_without_escapes = 0;
	register	char	*text_without_escapes;
	
	
	/* Berechne length_without_escapes	*/
	for (i = 0; i < length; i++) {
		if (text[i] == ESCAPE_CHARACTER)
			i ++;
		length_without_escapes ++;
	}

	/* Platz machen	*/
	text_without_escapes = (char *)malloc (length_without_escapes+1);
	
	/* Forme text in text_without_escapes um :	*/
	/* i = index in text;				*/
	/* j = index in text_without_escapes		*/
	for (i=0,j=0; i<length; i++,j++) {
		if (text[i] == ESCAPE_CHARACTER) {
			switch (text[i+1]) {
			    case 'n':
				text_without_escapes[j] = '\n';
				break;
			    case 'r':
				text_without_escapes[j] = '\r';
				break;
			    case 't':
				text_without_escapes[j] = '\t';
				break;
			    default:
				text_without_escapes[j] = text[i+1];
				break;
			}
			i++;
		} else {
			text_without_escapes[j] = text[i];
		}
	}
	text_without_escapes [length_without_escapes] = '\0';
	
	return	text_without_escapes;
}
